[llvm-commits] [llvm] r66167 - in /llvm/trunk: lib/Transforms/IPO/GlobalOpt.cpp test/Transforms/GlobalOpt/2009-03-03-dbg.ll

Devang Patel dpatel at apple.com
Thu Mar 5 10:12:03 PST 2009


Author: dpatel
Date: Thu Mar  5 12:12:02 2009
New Revision: 66167

URL: http://llvm.org/viewvc/llvm-project?rev=66167&view=rev
Log:
GlobalOpt only process non constant local GVs while optimizing global vars.

If non constant local GV named A is used by a constant local GV named B (e.g. llvm.dbg.variable) and B is not used by anyone else then eliminate A as well as B.

In other words, debug info should not interfere in removal of unused GV.
--This life, and those below, will be ignored--

M    test/Transforms/GlobalOpt/2009-03-03-dbg.ll
M    lib/Transforms/IPO/GlobalOpt.cpp

Modified:
    llvm/trunk/lib/Transforms/IPO/GlobalOpt.cpp
    llvm/trunk/test/Transforms/GlobalOpt/2009-03-03-dbg.ll

Modified: llvm/trunk/lib/Transforms/IPO/GlobalOpt.cpp
URL: http://llvm.org/viewvc/llvm-project/llvm/trunk/lib/Transforms/IPO/GlobalOpt.cpp?rev=66167&r1=66166&r2=66167&view=diff

==============================================================================
--- llvm/trunk/test/Transforms/GlobalOpt/2009-03-03-dbg.ll (original)
+++ llvm/trunk/test/Transforms/GlobalOpt/2009-03-03-dbg.ll Thu Mar  5 12:12:02 2009
@@ -1,5 +1,4 @@
 ; RUN: llvm-as < %s | opt -globalopt | llvm-dis | not grep global_variable42
-; XFAIL: *
 
 	%llvm.dbg.anchor.type = type { i32, i32 }
 	%llvm.dbg.basictype.type = type { i32, { }*, i8*, { }*, i32, i64, i64, i64, i32, i32 }





More information about the llvm-commits mailing list
--- llvm/trunk/lib/Transforms/IPO/GlobalOpt.cpp (original)
+++ llvm/trunk/lib/Transforms/IPO/GlobalOpt.cpp Thu Mar  5 12:12:02 2009
@@ -137,17 +137,28 @@
 }
 
 /// ConstantIsDead - Return true if the specified constant is (transitively)
-/// dead.  The constant may be used by other constants (e.g. constant arrays and
-/// constant exprs) as long as they are dead, but it cannot be used by anything
-/// else.
-static bool ConstantIsDead(Constant *C) {
+/// dead.  The constant may be used by other constants (e.g. constant arrays,
+/// constant exprs, constant global variables) as long as they are dead, 
+/// but it cannot be used by anything else. If DeadGVs is not null then
+/// record dead constant GV users.
+static bool ConstantIsDead(Constant *C, 
+                           SmallPtrSet<GlobalVariable *, 4> *DeadGVs = false) {
   if (isa<GlobalValue>(C)) return false;
 
-  for (Value::use_iterator UI = C->use_begin(), E = C->use_end(); UI != E; ++UI)
-    if (Constant *CU = dyn_cast<Constant>(*UI)) {
-      if (!ConstantIsDead(CU)) return false;
+  for (Value::use_iterator UI = C->use_begin(), E = C->use_end(); UI != E; ++UI) {
+    if (GlobalVariable *GV = dyn_cast<GlobalVariable>(*UI)) {
+      if (!GV->isConstant() || !GV->hasLocalLinkage() || !GV->use_empty())
+        return false;
+      else {
+        if (DeadGVs)
+          DeadGVs->insert(GV);
+      }
+    }
+    else if (Constant *CU = dyn_cast<Constant>(*UI)) {
+      if (!ConstantIsDead(CU, DeadGVs)) return false;
     } else
       return false;
+  }
   return true;
 }
 
@@ -338,8 +349,19 @@
     } else if (Constant *C = dyn_cast<Constant>(U)) {
       // If we have a chain of dead constantexprs or other things dangling from
       // us, and if they are all dead, nuke them without remorse.
-      if (ConstantIsDead(C)) {
-        C->destroyConstant();
+      SmallPtrSet<GlobalVariable *, 4> DeadGVs;
+      if (ConstantIsDead(C, &DeadGVs)) {
+        for (SmallPtrSet<GlobalVariable *, 4>::iterator TI = DeadGVs.begin(),
+               TE = DeadGVs.end(); TI != TE; ) {
+          GlobalVariable *TGV = *TI; ++TI;
+          if (TGV == C)
+            C = NULL;
+          TGV->eraseFromParent();
+        }
+        if (GlobalVariable *GV = dyn_cast<GlobalVariable>(C))
+          GV->eraseFromParent();
+        else if (C)
+          C->destroyConstant();
         // This could have invalidated UI, start over from scratch.
         CleanupConstantGlobalUsers(V, Init);
         return true;

Modified: llvm/trunk/test/Transforms/GlobalOpt/2009-03-03-dbg.ll
URL: http://llvm.org/viewvc/llvm-project/llvm/trunk/test/Transforms/GlobalOpt/2009-03-03-dbg.ll?rev=66167&r1=66166&r2=66167&view=diff

==============================================================================